// ============================================================================
// BDreaderDLPmain.cpp
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read "raw" file dump and extract the data with parameters.
// create .DLP file - level is packed with DeLuxePacker
// ============================================================================

#include "stdio.h"
#include <iostream>
#include <string>
#include <filesystem>

using namespace std;

unsigned char prog [0xFFFF+2] = {0};
unsigned char cave [880+100]  = {0};

char convert(unsigned char ch) {
    char retval= '?';
    switch(ch){
        case 0x10    : retval = 'o';  break;  // rock
        case 0x14    : retval = '*';  break;  // diamond
        case 0x03    : retval = 'm';  break;  // magicWall
        case 0x02    : retval = 'W';  break;  // wall
        case 0x07    : retval = '#';  break;  // steelWall
        case 0x2A    : retval = 'e';  break;  // expandWall
        case 0x3C    : retval = 'r';  break;  // rockford (fake like wall)
        case 0x01    : retval = ':';  break;  // dirt
        case 0x08    : retval = 'f';  break;  // firefly   (links)
        case 0x30    : retval = 'b';  break;  // butterfly
        case 0x3A    : retval = 'a';  break;  // amoeba
        case 0x3E    : retval = 's';  break;  // slime
        case 0x3D    : retval = 'H';  break;  // hiddenOut
        case 0x04    : retval = 'V';  break;  // visibleOut
        case 0x25    : retval = 'P';  break;  // player
        case 0x00    : retval = ' ';  break;  // space (empty)

        case 0x38    : retval = '@';  break;  // steelwall
        case 0x39    : retval = 'S';  break;  // slime
        case 0x57    : retval = 'A';  break;  // amoeba
        case 0x0B    : retval = '<';  break;  // ?? butterfly
        case 0x0A    : retval = '>';  break;  // ?? firefly rechts
        case 0x33    : retval = '3';  break;  // ??
        case 0x32    : retval = '2';  break;  // ??

        default      : retval = '?';  break;
    }
    return retval;
}

void outByte(unsigned char ch, bool init = false) {
    static int bx = 0;
    if (init) { 
        bx = ch;
    }
    if (!init) {
        printf("%2.2X ", ch);
        bx++;
        if(bx % 16 == 0) {
            printf("\n");
            printf("%8.8X  ", bx);
        }
    }
}

int main (int argc, char* argv[])
{
    bool doWrite = false;
    FILE* fpw = NULL;
    unsigned char ele = 0;
    unsigned char cnt = 0;
    int  maxread = 0x0BD8 - 0x0828;   // 0x03b0 = 944
    int  cavread = 880;
    int  pcs,pce,pcc;
    int  idx = 0;

    string path       = "C:\\temp\\BoulderDashTosec\\_d64\\";
    string fileprefix = "Arno Dash 21 (2014)(Arno)";
    string filename;

    fileprefix = "Demalion Dash 1 - Can You Dig It, Sucka (2010)(Liepa, P. & Tailor, D.)";
    if (argc > 1) {
        fileprefix = argv[1];
    }
    if (argc > 2) {
        doWrite = atoi (argv[2]);
    }

    filename = path + fileprefix + ".raw";

    FILE* fp = fopen(filename.c_str(), "rb");
    if (fp!=NULL){
        while(!feof(fp)){
            int rx  = 0;
            rx = fread(prog,1,0xFFFF,fp);
            if (rx == 0 || rx>=0xFFF0){
                break;
            }
        }
        fclose(fp);
    }

    if (doWrite) {
        filesystem::path dir1{path+fileprefix};
        filesystem::create_directory(dir1);
    }

    int eaddr;  // effektive addresse
    int maxy = 0;

    // cavetable lesen
    // 00 = normal
    // 01 = intermission
    int cavetableAddr  = 0x705F;
    unsigned char cavetable[100];
    memset(cavetable, 0xFF, sizeof(cavetable));
    for(int y=1;y<sizeof(cavetable);y++)
    {
        unsigned char byte;
        eaddr = cavetableAddr + y;
        byte  = prog[eaddr];
        if (byte == 0xFF) {
            break;
        }
        cavetable[y] = byte;
        maxy = y;
    }

    // offsettable lesen
    int offsetLowAddr  = 0x6FFF;
    int offsetHighAddr = 0x702F;
    int offsetTable[100] = {0};
    for(int y=1;y<=maxy;y++)
    {
        unsigned char lowbyte;
        unsigned char highbyte;
        eaddr = offsetLowAddr + y;
        lowbyte   = prog[eaddr];
        eaddr = offsetHighAddr + y;
        highbyte  = prog[eaddr];
        offsetTable[y] = highbyte * 256 + lowbyte;
    }

    // cave + para lesen
    for(int y=1;y<=maxy;y++)
    // for(int y=20;y<=20;y++)
    {
        unsigned char byte;
        eaddr = offsetTable[y];
        pcs   = eaddr;
        pce   = eaddr;
        pcc   = eaddr;
        memset(cave, 0, sizeof(cave));
        printf("=== %3d -- %4.4X\n\n", y, eaddr);
        printf("00000000  00 C4 44 4C 50 ");        // header // 00 C4 DLP
        
        // schreiben
        if (doWrite) {
            char wfile[256];
            sprintf(wfile, "%s\\%2.2d.DLP", (path + fileprefix).c_str(), y);
            fpw = fopen (wfile, "wb");
            if (fpw != NULL) {
                unsigned char buf[] = {0x00, 0xC4, 0x44, 0x4C, 0x50};
                fwrite(buf, 1, sizeof(buf), fpw);
            }
        }

        outByte(5, true);   // init
        idx   = 0;
        do {
            byte = prog[eaddr];
            outByte(byte);

            // schreiben
            if (doWrite && fpw != NULL) {
                unsigned char buf[] = {byte};
                fwrite(buf, 1, sizeof(buf), fpw);
            }

            if (byte == 0xBF) {
                ele = prog[eaddr + 1];
                cnt = prog[eaddr + 2];
                eaddr += 3;
                outByte(ele);
                outByte(cnt);
                // schreiben
                if (doWrite && fpw != NULL) {
                    unsigned char buf[] = {ele, cnt};
                    fwrite(buf, 1, sizeof(buf), fpw);
                }
            } else {
                ele = byte;
                cnt = 1;
                eaddr += 1;
            }
            pce = eaddr;

            for(int ic=1; ic<=cnt; ic++) {
                cave[idx] = ele;
                idx++;
            }
            if (idx == cavread) {
                pcc = pce;
            }
            if (idx >= maxread) {
                break;
            }
        } while(true);

        // schreiben
        if (doWrite && fpw != NULL) {
            if(fpw!=NULL) {
                fclose(fpw);
            }
        }

        printf("\n\n");
        // cave ausgeben
        int cc = 0;
        printf("%8.8X  ", cc);
        for (int yy=1;yy<=22;yy++) {
            for (int xx=1;xx<=40;xx++) {
                printf("%c", convert(cave[cc]));
                cc++;
            }
            printf("\n");
            printf("%8.8X  ", cc);
        }
        printf("\n");
        printf("\n");
        // para ausgeben
        for (int c=0;c<=0x3F;c++) {
            printf("%2.2x ", cave[cc]);
            cc++;
            if ((c+1)%16 == 0) {
                printf("\n");
            }
        }
        printf("\n");
    }
    printf("\n");
    printf("\n");
}
