// ============================================================================
// BDreaderGAMEBDmain.h
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read file dump from "BoulderDsh" (by Guido Mersmann) 
// and extract level data to -> ".gamebd" file
// generate "levelinfo.conf" with some extra info
// ============================================================================

#ifndef BDREADERGAMEBDMAIN_H
#define BDREADERGAMEBDMAIN_H

#include "BDreaderGAMEBDutil.h"

#define VIEWER_ONLY 1 

#define GAMEBD_MAGIC        "GAMEBD\0\0"

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

// normal elements (editor)
#define GAMEBD_EL_EMPTY             0x00    /* empty                    */
#define GAMEBD_EL_FAKEEMPTY         0x8B    /* fake empty               */
#define GAMEBD_EL_DIAMOND           0x80    /* diamond                  */
#define GAMEBD_EL_ROCK              0x02    /* rock                     */
#define GAMEBD_EL_FAKEROCK          0x0B    /* fake rock                */

#define GAMEBD_EL_EGG               0x40    /* egg                      */
#define GAMEBD_EL_DIRT              0x01    /* dirt                     */
#define GAMEBD_EL_FAKEDIRT          0x11    /* fake dirt                */
#define GAMEBD_EL_STEELWALL         0x04    /* steel wall               */
#define GAMEBD_EL_FAKESTEELWALL     0x0A    /* fake steel wall          */

#define GAMEBD_EL_WALL              0x05    /* wall                     */
#define GAMEBD_EL_MAGICWALL         0x85    /* magic wall               */
#define GAMEBD_EL_GROWWALLH         0x06    /* grow wall horizontal     */
#define GAMEBD_EL_GROWWALLV         0x07    /* grow wall vertical       */
#define GAMEBD_EL_WATER             0x82    /* water                    */

#define GAMEBD_EL_FIREFLY_DOWN      0xE2    /* firefly (down)           */
#define GAMEBD_EL_BUTTERFLY_UP      0xE5    /* butterfly (up)           */
#define GAMEBD_EL_PLAYER_IN         0x86    /* player in                */
#define GAMEBD_EL_VISIBLEOUT        0x87    /* visible out              */
#define GAMEBD_EL_HIDDENOUT         0x08    /* hidden out               */

#define GAMEBD_EL_FAKEOUT           0x88    /* visible out fake exit    */
#define GAMEBD_EL_TWIN              0x09    /* twin                     */
#define GAMEBD_EL_SLIME             0x83    /* slime                    */
#define GAMEBD_EL_GRAVITY_UP        0x0C    /* gravity up               */
#define GAMEBD_EL_GRAVITY_DOWN      0x0D    /* gravity down             */

#define GAMEBD_EL_GRAVITY_LEFT      0x0E    /* gravity left             */
#define GAMEBD_EL_GRAVITY_RIGHT     0x0F    /* gravity right            */
#define GAMEBD_EL_EARTHQUAKE        0x8A    /* earthquake               */
#define GAMEBD_EL_BLOCK_EMPTY       0x10    /* block empty              */

// special elements
#define GAMEBD_EL_DIAMOND_GHOST     0x38    /* diamond ghost (creating) */
#define GAMEBD_EL_TWIN_GHOST        0x89    /* twin ghost (fake)        */

#define GAMEBD_EL_FIREFLY_RIGHT     0xE0    /* firefly (right)          */
#define GAMEBD_EL_FIREFLY_LEFT      0xE1    /* firefly (left)           */
#define GAMEBD_EL_FIREFLY_UP        0xB2    /* firefly (up)             */

#define GAMEBD_EL_BUTTERFLY_LEFT    0xE3    /* butterfly (left)         */
#define GAMEBD_EL_BUTTERFLY_DOWN    0xE4    /* butterfly (down)         */
#define GAMEBD_EL_BUTTERFLY_RIGHT   0xB3    /* butterfly (right)        */

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// -- Anchor --
// kurz vorm EditorText ist 30 = editable  31 = not editable !!
// vorm Anchor ist noch: caveCurrentLevelAddr
//---------------------------------------------------------------------------
struct anchor_t {
    byte_t      caveFirstLevelAddr[4];
    byte_t      caveNullLevelAddr[4];       // 0000
    byte_t      caveLastLevelAddr[4];
    byte_t      editorPossible[4];          // 0100 dez // editor possible 0000
    byte_t      gameEditorText[6*16+4];
    byte_t      gameFileName[132];
};

//---------------------------------------------------------------------------
// -- CaveListHeader --
//---------------------------------------------------------------------------
struct caveListHeader_t {
    byte_t      caveNextLevelAddr[4];
    byte_t      caveLastLevelAddr[4];
    byte_t      unknownBytes01[2];          // 0000
    byte_t      caveStartAddr[4];
};

//---------------------------------------------------------------------------
// -- CaveHeader --
//---------------------------------------------------------------------------
struct caveHeader_t {
    byte_t      caveName[32+2];         //  "1234567890abcdefghij123456789012" 0000 
    byte_t      bdCave[6+2];            //  "bdCave"  0000
    byte_t      bdVersion[2];           //  0004
    byte_t      sizex[2];               //  
    byte_t      sizey[2];               //  

    byte_t      headsize[2];            //  0128   // always
    byte_t      levelLength[2];         //  + 2    
    byte_t      intermission[2];        //  + 4    // 0008 normal -> 000C intermission  (bits: 1000 1100) 002A 002E (1010 1110)
    byte_t      magicWallTime[2];       //  + 6    
    byte_t      levelTime[2];           //  + 8    
    byte_t      watergrowingtime[2];    //  +10    
    byte_t      watermaximum[1];        //  +12    
    byte_t      watergrowingrate[1];    //  +13    
    byte_t      diamondsNeeded[2];      //  +14    

    byte_t      unknown01[2];           //  +16    
    byte_t      diamondPoints[2];       //  +18    
    byte_t      unknown02[2];           //  +20    
    byte_t      diamondExtra[2];        //  +22    
    byte_t      unknown03[2];           //  +24    
    byte_t      diamondPoints99[2];     //  +26    
    byte_t      unknown04[2];           //  +28    
    byte_t      forBonusLife[2];        //  +30    

    byte_t      unknown05[2];           //  +32    
    byte_t      timeBonus[2];           //  +34    
    byte_t      gameSpeed[2];           //  +36    
    byte_t      gravity[2];             //  +38    // 0002=left 0004=right 0000=down 0001=up 
    byte_t      waterfloodrate[1];      //  +40    
    byte_t      slimerate[1];           //  +41    
    byte_t      unknown06[2];           //  +42    
    byte_t      unknown07[2];           //  +44    
    byte_t      magicwatertime[2];      //  +46    

    byte_t      earthQuake[2];          //  +48    // 0000=Off 0001=Shuffle 0002=Once 0003=Cycle 
    byte_t      activeTime[2];          //  +50    
    byte_t      earthDelay[2];          //  +52    
    byte_t      earthLength[2];         //  +54    
    byte_t      forceRate[1];           //  +56    
    byte_t      unknown08[1];           //  +57    
    byte_t      unknown09[2];           //  0000
    byte_t      unknown10[2];           //  0000
    byte_t      unknown11[2];           //  0000

    byte_t      unknown12[10];          //  0000 0000 0000 0000 0000

    byte_t      gfxName[32+2];
    byte_t      fntName[32+2];
    byte_t      typeName[32+2];
    byte_t      musName[32+2];
    byte_t      colorSetName[32+2];
    
    byte_t      levelScoreStartAddr[4];
};

//---------------------------------------------------------------------------
// -- Cave --
//---------------------------------------------------------------------------
struct cave_t {
    byte_t      cave[100*100+16+1];
};


//---------------------------------------------------------------------------
// -- CaveNew (for write) --
//---------------------------------------------------------------------------
struct caveNew_t {
    long            caveNextLevelAddr;
    long            caveLastLevelAddr;
    byte_t          unknownBytes01[2];
    long            caveStartAddr;
    //
    int             listHeaderLen;
    int             caveHeaderLen;
    int             caveLen;
    int             toLastByteLen;
    int             extraBytesLen;
    long            caveNewOffset;
    //
    caveHeader_t    caveheader;
    cave_t          cave;
    byte_t          extraBytes[16+1];
};

#endif
