// ============================================================================
// BDreaderGAMEBDoutput.cpp
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read file dump from "BoulderDsh" (by Guido Mersmann) 
// and extract level data to -> ".gamebd" file
// generate "levelinfo.conf" with some extra info
// ============================================================================

#include "BDreaderGAMEBDmain.h"
#include "BDreaderGAMEBDutil.h"
#include "BDreaderGAMEBDoutput.h"
#include <iostream>

using namespace std;


//---------------------------------------------------------------------------
// output Anchor 
//---------------------------------------------------------------------------
void outputAnchor(anchor_t &anchor) {
    outbytes("caveFirstLevelAddr   ", &anchor.caveFirstLevelAddr[0], sizeof(anchor.caveFirstLevelAddr));
    outbytes("caveNullLevelAddr    ", &anchor.caveNullLevelAddr[0],  sizeof(anchor.caveNullLevelAddr));
    outbytes("caveLastLevelAddr    ", &anchor.caveLastLevelAddr[0],  sizeof(anchor.caveLastLevelAddr));
    outbytes("editorPossible       ", &anchor.editorPossible[0],     sizeof(anchor.editorPossible));
    outtext ("gameEditorText       ", &anchor.gameEditorText[0],     sizeof(anchor.gameEditorText));
    outtext ("gameFileName         ", &anchor.gameFileName[0],       sizeof(anchor.gameFileName));
}

//---------------------------------------------------------------------------
// output CaveListHeader
//---------------------------------------------------------------------------
void outputCaveListHeader(caveListHeader_t &caveListHeader) {
    outbytes("caveNextLevelAddr    ", &caveListHeader.caveNextLevelAddr[0], sizeof(caveListHeader.caveNextLevelAddr));
    outbytes("caveLastLevelAddr    ", &caveListHeader.caveLastLevelAddr[0], sizeof(caveListHeader.caveLastLevelAddr));
    outbytes("unknownBytes01       ", &caveListHeader.unknownBytes01[0],    sizeof(caveListHeader.unknownBytes01));
    outbytes("caveNameAddr         ", &caveListHeader.caveStartAddr[0],     sizeof(caveListHeader.caveStartAddr));
}

//---------------------------------------------------------------------------
std::string earthquakeMode(long eqmode)
{
    if (eqmode == 0) {
        return std::string("Off");
    }
    if (eqmode == 1) {
        return std::string("Shuffle");
    }
    if (eqmode == 2) {
        return std::string("Once");
    }
    if (eqmode == 3) {
        return std::string("Cycle");
    }
    return std::string("????");
}

//---------------------------------------------------------------------------
std::string intermissionMode(long intmode)
{
    char hstr[40] = {0};
    long hmode = intmode & 0x0004;
    sprintf(hstr, "dec(%d) hex(%x) -> %s", intmode, intmode, (hmode>0) ? "on" : "off");
    std::string str = std::string (hstr);
    return str;
}

//---------------------------------------------------------------------------
std::string gravityMode(long gravmode)
{
    // 0002=left 0003=right 0000=down 0001=up 
    if (gravmode == 2) {
        return std::string("left");
    }
    if (gravmode == 3) {
        return std::string("right");
    }
    if (gravmode == 1) {
        return std::string("up");
    }
    if (gravmode == 0) {
        return std::string("down");
    }
    return std::string("????");
}

//---------------------------------------------------------------------------
// output CaveHeader
//---------------------------------------------------------------------------
void outputCaveHeader(caveHeader_t &caveHeader, int showOffset,
                      int showFilename, string filename, string gamename)
{
    cout << string(70, '-') << endl;
    printf ("================ Cave: %s", &caveHeader.caveName[0]);
    if (showFilename) {
        printf("\t -- %s\n", filename.c_str());
    } else {
        printf("\t -- %s\n", gamename.c_str());
    }
    // outtext ("== cave           ", &caveHeader.caveName[0], sizeof(caveHeader.caveName));
    cout << string(5,'-') << "Visual and Noisy Settings" << endl;
    cout << "== Level Width       : " << convBytes2offset(&caveHeader.sizex[0], sizeof(caveHeader.sizex)) << 
            "\t\tLevel Height     : " << convBytes2offset(&caveHeader.sizey[0], sizeof(caveHeader.sizey)) << endl;
    outtext ("Graphic Set       ", &caveHeader.gfxName[0], sizeof(caveHeader.gfxName));
    outtext ("Golor Set         ", &caveHeader.colorSetName[0], sizeof(caveHeader.colorSetName));
    outtext ("Character Set     ", &caveHeader.fntName[0], sizeof(caveHeader.fntName));
    outtext ("Sample Set        ", &caveHeader.typeName[0], sizeof(caveHeader.typeName));
    outtext ("Music             ", &caveHeader.musName[0], sizeof(caveHeader.musName));
    cout << string(5,'-') << "Water and Slime" << endl;
    cout << "--Water--" << endl;
    cout << "== Water Growing Time: " << convBytes2offset(&caveHeader.watergrowingtime[0], sizeof(caveHeader.watergrowingtime)) <<
            "\t\tMagic Water Time : " << convBytes2offset(&caveHeader.magicwatertime[0], sizeof(caveHeader.magicwatertime)) << endl;
    cout << "== Water Growing Rate: " << convBytes2offset(&caveHeader.watergrowingrate[0], sizeof(caveHeader.watergrowingrate)) <<
            "\t\tWater Flood Rate : " << convBytes2offset(&caveHeader.waterfloodrate[0], sizeof(caveHeader.waterfloodrate)) << endl;
    cout << "== Water Maximum     : " << convBytes2offset(&caveHeader.watermaximum[0], sizeof(caveHeader.watermaximum)) << endl;
    cout << "--Slime--" << endl;
    cout << "== Slime Rate        : " << convBytes2offset(&caveHeader.slimerate[0], sizeof(caveHeader.slimerate)) << endl;
    cout << string(5,'-') << "Earthquake" << endl;
    cout << "== Earthquake Mode   : " << earthquakeMode(convBytes2offset(&caveHeader.earthQuake[0], sizeof(caveHeader.earthQuake))) << endl;

    long tmpActiveTime    = convBytes2offset(&caveHeader.activeTime[0], sizeof(caveHeader.activeTime));
    long tmpDelay         = convBytes2offset(&caveHeader.earthDelay[0], sizeof(caveHeader.earthDelay));
    long tmpEarthLength   = convBytes2offset(&caveHeader.earthLength[0], sizeof(caveHeader.earthLength));
    char strActiveTime[20];
    char strDelay[20];
    char strEarthLength[20];
    sprintf(strActiveTime,  "%s", (tmpActiveTime == 0)  ? "0 -> 1" : tostr(tmpActiveTime).c_str());
    sprintf(strDelay,       "%s", (tmpDelay == 0)       ? "0 -> 1" : tostr(tmpDelay).c_str());
    sprintf(strEarthLength, "%s", (tmpEarthLength == 0) ? "0 -> 1" : tostr(tmpEarthLength).c_str());

    cout << "== Activate Time     : " << strActiveTime  <<
            "\t\tDelay            : " << strDelay       << endl;
    cout << "== Force Rate        : " << convBytes2offset(&caveHeader.forceRate[0], sizeof(caveHeader.forceRate))  << 
            "\t\tLength           : " << strEarthLength << endl;
    cout << string(5,'-') << "Points" << endl;
    cout << "== Diamond           : " << convBytes2offset(&caveHeader.diamondPoints[0], sizeof(caveHeader.diamondPoints)) << 
            "\t\tDiamond Extra    : " << convBytes2offset(&caveHeader.diamondExtra[0], sizeof(caveHeader.diamondExtra)) << endl;
    cout << "== Diamond >99       : " << convBytes2offset(&caveHeader.diamondPoints99[0], sizeof(caveHeader.diamondPoints99)) << 
            "\t\tFor Bonus Life   : " << convBytes2offset(&caveHeader.forBonusLife[0], sizeof(caveHeader.forBonusLife)) << endl;
    cout << "== Time Bonus        : " << convBytes2offset(&caveHeader.timeBonus[0], sizeof(caveHeader.timeBonus)) << endl;
    cout << string(5,'-') << "Miscellaneous" << endl;
    cout << "== Game Speed        : " << convBytes2offset(&caveHeader.gameSpeed[0], sizeof(caveHeader.gameSpeed)) << endl;
    cout << "== Level Time        : " << convBytes2offset(&caveHeader.levelTime[0], sizeof(caveHeader.levelTime)) << endl;
    cout << "== Magic Wall Time   : " << convBytes2offset(&caveHeader.magicWallTime[0], sizeof(caveHeader.magicWallTime)) << endl;
    cout << "== Diamonds Needed   : " << convBytes2offset(&caveHeader.diamondsNeeded[0], sizeof(caveHeader.diamondsNeeded)) << endl;
    cout << "== Intermission      : " << intermissionMode(convBytes2offset(&caveHeader.intermission[0], sizeof(caveHeader.intermission))) << endl;
    cout << "== Gravitation Direct: " << gravityMode(convBytes2offset(&caveHeader.gravity[0], sizeof(caveHeader.gravity))) << endl;

    if (showOffset) {
        cout << string(5,'-') << "LevelScoreStartAddr" << endl;
        cout << " -> " << dec2hex(bytes2offset(&caveHeader.levelScoreStartAddr[0])) << endl;
    }

    cout << string(70, '-') << endl;
}
