// ============================================================================
// BDreaderGAMEBDoutputCaveGAMEBD.cpp
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read file dump from "BoulderDsh" (by Guido Mersmann) 
// and extract level data to -> ".gamebd" file
// generate "levelinfo.conf" with some extra info
// ============================================================================

#include "BDreaderGAMEBDmain.h"
#include "BDreaderGAMEBDoutputCaveGAMEBD.h"
#include <map>


/*------------------------------------------------------------------------------------------------------
-- elements in editor sorted
--------------------------------------------------------------------------------------------------------
----------------------------------00 Empty
----------------------------------------8B FakeEmpty 
----------------------------------------------80 Diamond
----02 Rock
----------0B FakeRock
----------------40 Egg
----------------------01 Dirt
----------------------------11 FakeDirt
----------------------------------04 SteelWall
----------------------0A FakeMetal
----------------------------05 Wall
----------------------------------85 MagicWall
----------------------------------------06 GrowingWallHorizontal
----------------------------------------------07 GrowingWallVertical
----82 Water
----------E2 Firefly
----------------E5 Butterfly
----------------------86 PlayerIN
----------87 VisibleOUT
----------------08 HiddenOUT 
----------------------88 VisibleOutFakeExit
----------------------------09 Twin
----------------------------------83 Slime
----------------------------------------0C Gravity Up
----------------------------------------------0D Gravity Down
----0E Gravity Left
----------0F Gravity Right
----------------------------------------------8A Earthquake
----10 BlockEmpty
------------------------------------------------------------------------------------------------------
    FF 04 29 FF 01 13 FF 04 15 01 00 01 8B 01 80 01
    02 01 0B 01 40 01 01 01 11 01 04 01 FF 04 15 FF
    01 13 FF 04 15 01 0A 01 05 01 85 01 06 01 07 01
    82 01 E2 01 E5 01 86 01 FF 04 15 FF 01 13 FF 04
    15 01 87 01 08 01 88 01 09 01 83 01 0C 01 0D 01
    0E 01 0F 01 FF 04 15 FF 01 13 FF 04 15 01 8A 01
    10 FF 01 0F FF 04 15 FF 01 13 FF 04 FF FF 04 F5
    FF FF 00 00 00 00 00 0D C6 98 00 00 00 C0 00 00
------------------------------------------------------------------------------------------------------*/

static std::map<int, std::string>  convertMap;

//---------------------------------------------------------------------------
// add to GAMEBD convert map 
//---------------------------------------------------------------------------
static void convertMapAdd(int ele, std::string str)
{
    convertMap[ele] = str;
}

//---------------------------------------------------------------------------
// init GAMEBD convert map 
//---------------------------------------------------------------------------
static void convertMapInit()
{
    convertMap.clear();
    // normal elements
    convertMapAdd(GAMEBD_EL_EMPTY            , " ");
    convertMapAdd(GAMEBD_EL_FAKEEMPTY        , "e");
    convertMapAdd(GAMEBD_EL_DIAMOND          , "*");
    convertMapAdd(GAMEBD_EL_ROCK             , "o");
    convertMapAdd(GAMEBD_EL_FAKEROCK         , "r");
    
    convertMapAdd(GAMEBD_EL_EGG              , "N");
    convertMapAdd(GAMEBD_EL_DIRT             , ".");
    convertMapAdd(GAMEBD_EL_FAKEDIRT         , ":");
    convertMapAdd(GAMEBD_EL_STEELWALL        , "#");
    convertMapAdd(GAMEBD_EL_FAKESTEELWALL    , "=");
    
    convertMapAdd(GAMEBD_EL_WALL             , "W");
    convertMapAdd(GAMEBD_EL_MAGICWALL        , "M");
    convertMapAdd(GAMEBD_EL_GROWWALLH        , ">");
    convertMapAdd(GAMEBD_EL_GROWWALLV        , "^");
    convertMapAdd(GAMEBD_EL_WATER            , "~");
    
    convertMapAdd(GAMEBD_EL_FIREFLY_DOWN     , "F");
    convertMapAdd(GAMEBD_EL_BUTTERFLY_UP     , "B");
    convertMapAdd(GAMEBD_EL_PLAYER_IN        , "P");
    convertMapAdd(GAMEBD_EL_VISIBLEOUT       , "!");
    convertMapAdd(GAMEBD_EL_HIDDENOUT        , "$");
    
    convertMapAdd(GAMEBD_EL_FAKEOUT          , "&");
    convertMapAdd(GAMEBD_EL_TWIN             , "T");
    convertMapAdd(GAMEBD_EL_SLIME            , "S");
    convertMapAdd(GAMEBD_EL_GRAVITY_UP       , "U");
    convertMapAdd(GAMEBD_EL_GRAVITY_DOWN     , "D");
    
    convertMapAdd(GAMEBD_EL_GRAVITY_LEFT     , "L");
    convertMapAdd(GAMEBD_EL_GRAVITY_RIGHT    , "R");
    convertMapAdd(GAMEBD_EL_EARTHQUAKE       , "Q");
    convertMapAdd(GAMEBD_EL_BLOCK_EMPTY      , "B");
    
    // special elements
    convertMapAdd(GAMEBD_EL_DIAMOND_GHOST    , "+");
    convertMapAdd(GAMEBD_EL_TWIN_GHOST       , "t");
    
    convertMapAdd(GAMEBD_EL_FIREFLY_RIGHT    , "f");
    convertMapAdd(GAMEBD_EL_FIREFLY_LEFT     , "f");
    convertMapAdd(GAMEBD_EL_FIREFLY_UP       , "f");
    
    convertMapAdd(GAMEBD_EL_BUTTERFLY_LEFT   , "b");
    convertMapAdd(GAMEBD_EL_BUTTERFLY_DOWN   , "b");
    convertMapAdd(GAMEBD_EL_BUTTERFLY_RIGHT  , "b");
}

#define ELEMENT_NOT_FOUND     "?"

//---------------------------------------------------------------------------
// convert GAMEBD to displayable char 
//---------------------------------------------------------------------------
std::string convertGAMEBD(int x, int y, int ch, bool &elFound)
{
    std::string el = ELEMENT_NOT_FOUND;
    if (convertMap.size() == 0) {
        convertMapInit();
    }
    char elHex[40] = {0};

    sprintf(elHex, "x: %d y: %d ch: %2.2x", x, y, ch);           // x, y, ch // for debugging

    std::map<int, std::string>::const_iterator it = convertMap.find(ch);
    if (it!=convertMap.end()) {
        el = it->second;
        elFound = true;
    } else {
        el = ELEMENT_NOT_FOUND;
        elFound = false;
    }
    return el;
}



