// ============================================================================
// BDreaderGAMEBDoutputCaveMagic64.cpp
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read file dump from "BoulderDsh" (by Guido Mersmann) 
// and extract level data to -> ".gamebd" file
// generate "levelinfo.conf" with some extra info
// ============================================================================

#include "BDreaderGAMEBDmain.h"
#include "BDreaderGAMEBDoutputCaveMagic64.h"
#include <map>

/*------------------------------------------------------------------------------------------------------
-- elements from BD64
/*------------------------------------------------------------------------------------------------------
/*--
        switch (ch) {
        case  0 : el = EL_BD_ROCK;                   // boulder 
                    break;
        case  1 : el = EL_BD_DIAMOND;                // diamond 
                    break;
        case  2 : el = EL_BD_MAGIC_WALL;             // magic wall 
                    break;
        case  3 : el = EL_BD_WALL;                   // brick wall 
                    break;
        case  4 : el = EL_STEELWALL;                 // steel wall 
                    break;
        case  5 : el = EL_BD_EXPANDABLE_WALL;        // expand wall 
                    break;
        case  6 : el = EL_PLAYER_1;                  // rockford 
                    break;
        case  7 : el = EL_SAND;                      // dirt
                    break;
        case  8 : el = EL_BD_FIREFLY;                // firefly
                    break;
        case  9 : el = EL_BD_BUTTERFLY;              // butterfly 
                    break;
        case 10 : el = EL_BD_AMOEBA;                 // amoeba 
                    break;
        case 11 : el = EL_AMOEBA_WET;                // slime
                    break;
        case 12 : el = EL_EM_STEEL_EXIT_CLOSED;      // hidden out 
                    break;
        case 13 : el = EL_EM_EXIT_CLOSED;            // visible out
                    break;
        case 14 : el = EL_PLAYER_1;                  // in 
                    break;
        case 15 : el = EL_EMPTY;                     // space
                    break;
--*/
/*----------------------------------------------------------------------------------------------------*/

static std::map<int, std::string>  convertMap;

//---------------------------------------------------------------------------
// add to GAMEBD convert map 
//---------------------------------------------------------------------------
static void convertMapAdd(int ele, std::string str)
{
    convertMap[ele] = str;
}

//---------------------------------------------------------------------------
// init GAMEBD convert map 
//---------------------------------------------------------------------------
static void convertMapInit()
{
    convertMap.clear();
    // normal elements
    convertMapAdd(GAMEBD_EL_EMPTY            , "00");    // empty
    convertMapAdd(GAMEBD_EL_FAKEEMPTY        , "--");    // --
    convertMapAdd(GAMEBD_EL_DIAMOND          , "14");    // diamond
    convertMapAdd(GAMEBD_EL_ROCK             , "10");    // rock
    convertMapAdd(GAMEBD_EL_FAKEROCK         , "--");    // --
    
    convertMapAdd(GAMEBD_EL_EGG              , "--");    // --
    convertMapAdd(GAMEBD_EL_DIRT             , "01");    // dirt
    convertMapAdd(GAMEBD_EL_FAKEDIRT         , "--");    // --
    convertMapAdd(GAMEBD_EL_STEELWALL        , "07");    // steel wall
    convertMapAdd(GAMEBD_EL_FAKESTEELWALL    , "--");    // --
    
    convertMapAdd(GAMEBD_EL_WALL             , "02");    // wall
    convertMapAdd(GAMEBD_EL_MAGICWALL        , "++");    // magic wall
    convertMapAdd(GAMEBD_EL_GROWWALLH        , "++");    // grow
    convertMapAdd(GAMEBD_EL_GROWWALLV        , "++");    // grow
    convertMapAdd(GAMEBD_EL_WATER            , "++");    // water // amoeba
    
    convertMapAdd(GAMEBD_EL_FIREFLY_DOWN     , "08");    // firefly
    convertMapAdd(GAMEBD_EL_BUTTERFLY_UP     , "09");    // butterfly
    convertMapAdd(GAMEBD_EL_PLAYER_IN        , "38");    // rockford // playerIn 
    convertMapAdd(GAMEBD_EL_VISIBLEOUT       , "04");    // visible out
    convertMapAdd(GAMEBD_EL_HIDDENOUT        , "++");    // hidden out
    
    convertMapAdd(GAMEBD_EL_FAKEOUT          , "--");    // --
    convertMapAdd(GAMEBD_EL_TWIN             , "--");    // --
    convertMapAdd(GAMEBD_EL_SLIME            , "++");    // slime
    convertMapAdd(GAMEBD_EL_GRAVITY_UP       , "--");    // --
    convertMapAdd(GAMEBD_EL_GRAVITY_DOWN     , "--");    // --
    
    convertMapAdd(GAMEBD_EL_GRAVITY_LEFT     , "--");    // --
    convertMapAdd(GAMEBD_EL_GRAVITY_RIGHT    , "--");    // --
    convertMapAdd(GAMEBD_EL_EARTHQUAKE       , "--");    // --
    convertMapAdd(GAMEBD_EL_BLOCK_EMPTY      , "--");    // --
    
    // special elements
    convertMapAdd(GAMEBD_EL_DIAMOND_GHOST    , "--");    // --
    convertMapAdd(GAMEBD_EL_TWIN_GHOST       , "--");    // --
    
    convertMapAdd(GAMEBD_EL_FIREFLY_RIGHT    , "--");    // --
    convertMapAdd(GAMEBD_EL_FIREFLY_LEFT     , "--");    // --
    convertMapAdd(GAMEBD_EL_FIREFLY_UP       , "--");    // --
    
    convertMapAdd(GAMEBD_EL_BUTTERFLY_LEFT   , "--");    // --
    convertMapAdd(GAMEBD_EL_BUTTERFLY_DOWN   , "--");    // --
    convertMapAdd(GAMEBD_EL_BUTTERFLY_RIGHT  , "--");    // --
}


#define ELEMENT_NOT_FOUND     "??"

//---------------------------------------------------------------------------
// convert GAMEBD to displayable char 
//---------------------------------------------------------------------------
std::string convertMagic64(int x, int y, int ch, bool &elFound)
{
    std::string el = ELEMENT_NOT_FOUND;
    if (convertMap.size() == 0) {
        convertMapInit();
    }
    char elHex[40] = {0};

    sprintf(elHex, "x: %d y: %d ch: %2.2x", x, y, ch);           // x, y, ch // for debugging

    std::map<int, std::string>::const_iterator it = convertMap.find(ch);
    if (it!=convertMap.end()) {
        el = it->second;
        elFound = true;
    } else {
        el = ELEMENT_NOT_FOUND;
        elFound = false;
    }
    return el;
}

