// ============================================================================
// BoulderDaeshReader -- util.h
// ----------------------------------------------------------------------------
// (c) 2022-2023 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read file dump from "BoulderDsh" (by Guido Mersmann) 
// and extract level data to -> ".gamebd" file
// generate "levelinfo.conf" with some extra info
// ============================================================================

#ifndef UTIL_H
#define UTIL_H

#include <stdio.h>
#include <cstdint>
#include <iomanip>
#include <sstream>

//---------------------------------------------------------------------------
typedef uint8_t         byte_t;
typedef uint16_t        word_t;
typedef uint32_t        dword_t;
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// handleError (for debugging)
//---------------------------------------------------------------------------
void handleError(const char * errMsg);


//---------------------------------------------------------------------------
// dec2hex -- convert long to hex (fill with zero)
//---------------------------------------------------------------------------
std::string dec2hex(long dec);

//---------------------------------------------------------------------------
// hex2dec -- convert hex to long
//---------------------------------------------------------------------------
long hex2dec(std::string hex_value);

//---------------------------------------------------------------------------
// outbytes -- print title -- then hex values (maxlen is parameter)
// blocksize is 16 -- bundlesize is 1
//---------------------------------------------------------------------------
void outbytes (const char *title, byte_t *what, size_t len,
               size_t bundlesize = 1, size_t blocksize = 16);

//---------------------------------------------------------------------------
// outtext -- print title -- then text (maxlen is parameter)
//---------------------------------------------------------------------------
void outtext (const char *title, byte_t *what, size_t len);

//---------------------------------------------------------------------------
// convBytes2offset -- convert bytes to long (size is parameter)
//---------------------------------------------------------------------------
long convBytes2offset (byte_t *what, size_t len);

//---------------------------------------------------------------------------
// bytes2offset -- convert bytes to long (default size is 4)
//---------------------------------------------------------------------------
long bytes2offset (byte_t *what);

//---------------------------------------------------------------------------
// tostr -- convert long to string - with "_ltoa" - basis 10
//---------------------------------------------------------------------------
std::string tostr(long w);

//---------------------------------------------------------------------------
// tohexstr -- convert long to string - with "_ltoa" - basis 16
// fill with zeros (default length 4)
//---------------------------------------------------------------------------
std::string tohexstr(long w, size_t sz = 4);

//---------------------------------------------------------------------------
// HexToBytes -- convert hexstr to bytes (sz is size of bytes)
//---------------------------------------------------------------------------
void HexToBytes(byte_t *bytes, const char* hexstr, size_t sz);

//---------------------------------------------------------------------------
// DecToBytes -- convert long to bytes (sz is size of bytes)
//---------------------------------------------------------------------------
void DecToBytes(byte_t *bytes, long deci, size_t sz);

#endif
