// ============================================================================
// BDreaderOTOmain.cpp
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read "raw" file dump and extract the data with parameters.
// create .OTO file - level is stored 1:1
// ============================================================================

#include "stdio.h"
#include <iostream>
#include <string>
#include <filesystem>

using namespace std;

unsigned char prog [0xFFFF+2] = {0};
unsigned char cave [880+100]  = {0};

/* Creature code conversion table */
/* Converts the C64 BoulderDash codes into the codes used by Jeff Bevis's
   Amiga implementation of BoulderDash. */
static unsigned char creatureCode[64]=
{ //00   01   02   03   04   05   06   07
    ' ', '.', 'w', 'm', 'P', 'P', '?', 'W',
  //08   09   0A   0B   0C   0D   0E   0F
    'q', 'o', 'Q', 'O', 'q', 'o', 'Q', 'O',
  //10   11   12   13   14   15   16   17
    'r', 'r', 'r', 'r', 'd', 'd', 'd', 'd',
  //18   19   1A   1B   1C   1D   1E   1F
    '?', '?', '?', '?', '?', '?', '?', '?',
  //20   21   22   23   24   25   26   27
    '?', '?', '?', '?', '3', 'X', '5', '6',
  //28   29   2A   2B   2C   2D   2E   2F
    '?', '?', 'E', '?', '?', '?', '?', '?',         // 2A
  //30   31   32   33   34   35   36   37
    'B', 'c', 'b', 'C', 'B', 'c', 'b', 'C',
  //38   39   3A   3B   3C   3D   3E   3F           // 3E
    '7', '8', 'a', 'a', '?', '?', 'S', '?' 
};

char convert(int cave, int x, int y, unsigned char ch) {
    char retval= '?';
    if (ch >= 0 && ch < 64) {
        retval = creatureCode[ch];
    } else {
        retval = '?';
    }
    if (retval == '?') 
    {
        switch(ch){
            /*-- should be in "creatureCode"
            case 0x10    : retval = 'o';  break;  // rock
            case 0x14    : retval = '*';  break;  // diamond
            case 0x03    : retval = 'm';  break;  // magicWall
            case 0x02    : retval = 'W';  break;  // wall
            case 0x07    : retval = '#';  break;  // steelWall
            case 0x2A    : retval = 'e';  break;  // expandWall
            case 0x3C    : retval = 'r';  break;  // rockford (fake like wall)
            case 0x01    : retval = ':';  break;  // dirt
            case 0x08    : retval = 'f';  break;  // firefly   (links)
            case 0x30    : retval = 'b';  break;  // butterfly
            case 0x3A    : retval = 'a';  break;  // amoeba
            case 0x3E    : retval = 's';  break;  // slime
            case 0x3D    : retval = 'H';  break;  // hiddenOut
            case 0x04    : retval = 'V';  break;  // visibleOut
            case 0x25    : retval = 'P';  break;  // player
            case 0x00    : retval = ' ';  break;  // space (empty)
            --*/

            case 0x38    : retval = '@';  break;  // steelwall
            case 0x39    : retval = 'S';  break;  // slime
            case 0x57    : retval = 'A';  break;  // amoeba

            case 0x0B    : retval = '<';  break;  // ?? butterfly
            case 0x0A    : retval = '>';  break;  // ?? firefly rechts
            case 0x33    : retval = '3';  break;  // ??

            case 0x56    : retval = 'M';  break;  // mrs.rockford
            case 0x6D    : retval = 'K';  break;  // key 
            case 0x6E    : retval = 'D';  break;  // door 
            case 0x78    : retval = 'J';  break;  // jamjam

            case 0x6C    : retval = 'D';  break;  // diamond blocked not eatable
            case 0x6B    : retval = 'B';  break;  // rock blocked
            case 0x64    : retval = 'G';  break;  // pacman ghost
            case 0x3B    : retval = 'N';  break;  // bonbon
            case 0x40    : retval = 'C';  break;  // classes (water bubbles)

            case 0x32    : retval = '2';  break;  // ?? butterfly
            case 0x29    : retval = 'Q';  break;  // ?? mouse
            case 0x2E    : retval = 'h';  break;  // ?? hidden
            case 0x1E    : retval = 'T';  break;  // ?? time
            case 0x06    : retval = '6';  break;  // ?? 

            default      : retval = '?';  break;
        }
    }
    return retval;
}

void outByte(unsigned char ch, bool init = false) {
    static int bx = 0;
    if (init) { 
        bx = ch;
    }
    if (!init) {
        printf("%2.2X ", ch);
        bx++;
        if(bx % 16 == 0) {
            printf("\n");
            printf("%8.8X  ", bx);
        }
    }
}

int main (int argc, char* argv[])
{
    bool doWrite = false;
    unsigned char ele = 0;
    unsigned char cnt = 0;
    int  maxread = 0x83BD - 0x8010 + 1;   // 0x03AE = 942
    int  maxpara = 0x83BD - 0x8380 + 1;   // 0x3E   = 62
    int  cavread = 880;
    int  pcs,pce,pcc;
    int  idx = 0;

    string path       = "C:\\temp\\BoulderDashTosec\\_d64\\";
    string fileprefix;
    string filename;

    fileprefix = "Bilder Dash (1994)(Peter Liepa & Olaf Roth)";
    //fileprefix = "Bilder Dash (1994)(Peter Liepa & Olaf Roth)[a]";
    //fileprefix = "Cool Dash #01 (1992)(Peter Liepa & Olaf Roth)";

    if (argc > 1) {
        fileprefix = argv[1];
    }
    if (argc > 2) {
        doWrite = atoi(argv[2]);
    }

    filename = path + fileprefix + ".raw";

    printf("== read file: %s\n", filename.c_str());

    FILE* fp = fopen(filename.c_str(), "rb");
    if (fp!=NULL){
        while(!feof(fp)){
            int rx  = 0;
            rx = fread(prog,1,0xFFFF,fp);
            if (rx == 0 || rx>=0xFFF0){
                break;
            }
        }
        fclose(fp);
    }

    FILE* fpw = NULL;
    int eaddr;  // effektive addresse
    int maxCaveNo = 0x15;

    // cave + para lesen
    for(int caveNo=1;caveNo<maxCaveNo;caveNo++)
    //for(int caveNo=5;caveNo<=5;caveNo++)
    {
        unsigned char byte;
        eaddr = 0x7010 + 4*(caveNo-1)*256;
        pcs   = eaddr;
        pce   = eaddr;
        pcc   = eaddr;
        memset(cave, 0, sizeof(cave));
        printf("=== %3d -- %4.4X\n\n", caveNo, eaddr);
        printf("00000000  00 C0 4F 54 4F ");        // header // 
        
        //
        if (doWrite) {
            filesystem::path dir1{path+fileprefix};
            filesystem::create_directory(dir1);
        }

        // schreiben
        if (doWrite) {
            char wfile[256];
            sprintf(wfile, "%s\\%2.2d.OTO", (path + fileprefix).c_str(), caveNo);
            fpw = fopen (wfile, "wb");
            if (fpw != NULL) {
                unsigned char buf[] = {0x00, 0xC0, 'O', 'T', 'O'};
                fwrite(buf, 1, sizeof(buf), fpw);
            }
        }

        outByte(5, true);   // init
        idx   = 0;
        do {
            byte = prog[eaddr];
            outByte(byte);

            // schreiben
            if (doWrite) {
                if (fpw != NULL) {
                    unsigned char buf[] = {byte};
                    fwrite(buf, 1, sizeof(buf), fpw);
                }
            }

            ele = byte;
            cnt = 1;
            eaddr += 1;

            pce = eaddr;

            for(int ic=1; ic<=cnt; ic++) {
                cave[idx] = ele;
                idx++;
            }
            if (idx == cavread) {
                pcc = pce;
            }
            if (idx >= maxread) {    
                break;
            }
        } while(true);

        // schreiben
        if (doWrite) {
            if(fpw!=NULL) {
                fclose(fpw);
            }
        }

        printf("\n\n");
        // cave ausgeben
        int cc = 0;
        printf("%8.8X  ", cc);
        for (int yy=1;yy<=22;yy++) {
            for (int xx=1;xx<=40;xx++) {
                printf("%c", convert(caveNo, xx, yy, cave[cc]));
                cc++;
            }
            printf("\n");
            printf("%8.8X  ", cc);
        }
        printf("\n");
        printf("\n");
        // para ausgeben
        for (int c=0;c<maxpara;c++) {   // 62 bytes
            printf("%2.2x ", cave[cc]);
            cc++;
            if ((c+1)%16 == 0) {
                printf("\n");
            }
        }
        printf("\n");
    }
    printf("\n");
    printf("\n");
}
