// ============================================================================
// BDreaderWLCcommonMain.cpp
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read "raw" file dump and extract the data with parameters.
// create .WLC file - WithLevelCommon
// ----------------------------------------------------------------------------
// In memory of Peter Broadripp's - He wrote "docodeCave.c"
// ----------------------------------------------------------------------------
// I fix some bugs - DrawFilledRect, SteelBounds before Commands,
//                 - MemoryAccess to the CaveData
// and do small improvements.
// -> cave compare now deliver no differents  
// ----------------------------------------------------------------------------
// implement caveReader for BoulderDash02 (BD02)
// ----------------------------------------------------------------------------
// HerzAusGold :)
// ============================================================================

#include "stdio.h"
#define NDEBUG /* Uncomment if you don't want assertion */
#include "assert.h"
#include <iostream>
#include <string>
#include "decodeCave.h"
#include "decodeCaveCommon.h"
#include "decodeCaveBD02.h"


using namespace std;

unsigned char prog [0xFFFF+2]   = {0};
/*----------------------------------------------------------------------------------------------------------*/
unsigned char cave [40*24+10]   = {0};
unsigned char para [0xFF+10]    = {0};


/* Output file */
FILE *theFile;


/*----------------------------------------------------------------------------------------------------------*/
void outByte(unsigned char ch, bool init = false, int bxinit = 0) {
    static int bx = 0;
    if (init) { 
        bx = bxinit;
    }
    if (!init) {
        printf("%2.2X ", ch);
        bx++;
        if(bx % 16 == 0) {
            printf("\n");
            printf("%8.8X  ", bx);
        }
    } else {
        printf("%8.8X  ", bx);
    }
}


/*============================================================================================================*/
int main ()
{
    int  maxread = 880;
    int  cavread = 880;
    int  offpara = 0x20;   // offset
    int  maxpara = 0xFF;   // all with commands 
    int  cidx = 0;
    int  pidx = 0;
    int  doDecodeCave   = 0;  // common

    int caveMax = 0;
    string filename;
    string fileprefix;
    string path;
    path = "C:\\temp\\BoulderDashTosec\\_d64\\";

    /*--
    fileprefix = "Boulder Change (1987)(Liepa, P.)";    //--OK--//
    caveMax = 20;
    --*/

    /*--
    fileprefix = "Boulder Dash - Rocky (1984)(First Star)(es)[h Load 'n' Run]";     //--OK--//  // seems BD01
    caveMax = 20;
    --*/

    /*--
    fileprefix = "Boulder Dash (1984)(First Star)[cr Nova]";    //--OK--//  // is BD01
    caveMax = 20;
    --*/

    /*--
    fileprefix = "Boulder Dash 01 (1984)(First Star)[cr Fall Guys]";    //--OK--//  // Boulderdash IV -> but this is not Boulderdash04 !! and not BD01 !!
    caveMax = 20;
    --*/

    /*--
    fileprefix = "Boulder Dash 02 (1986)(First Star)";      //--OK--//
    caveMax = 20;
    offpara = 0x1A;
    doDecodeCave   = 1;
    --*/

    /*--
    fileprefix = "Boulder Dash 03 (1986)(First Star)";      //--OK--//
    caveMax = 20;
    --*/

    /*--
    fileprefix = "Boulder Dash 04 (1986)(First Star)";      //--OK--//
    caveMax = 20;
    /*--*/

    /*--
    fileprefix = "De Luxe Dash 3 (1990)(Peter Liepa & Marek Roth)[a]--DREAM3";  //--OK--//  // need special feature
    caveMax = 20;
    --*/

    /*--
    fileprefix = "Profi Boulder 043 (1992)(Peter Liepa & Rockford)";    //--OK--//
    caveMax = 20;
    offpara = 0x1A;
    doDecodeCave   = 1;
    --*/

    fileprefix = "Profi Boulder 087 (1994)(Peter Liepa & Rockford)";
    caveMax = 20;
    offpara = 0x1A;
    doDecodeCave   = 1;

    //----------------------------------------------------------------------------------------
    DecodeCave* decodeCave = NULL;
    switch(doDecodeCave) {
        case 0 : decodeCave = new DecodeCaveCommon;      
                 ((DecodeCaveCommon*)decodeCave)->SetFeature(c_feature_command0x0F, 1);
                 break;
        case 1 : decodeCave = new DecodeCaveBD02;        
                 break;
    }

    filename = path + fileprefix + ".raw";
    FILE* fp = fopen(filename.c_str(), "rb");
    if (fp!=NULL){
        while(!feof(fp)){
            int rx  = 0;
            rx = fread(prog,1,0xFFFF,fp);
            if (rx == 0 || rx>=0xFFF0){
                break;
            }
        }
        fclose(fp);
    } else {
        printf("-E- File not found: %s\n", filename.c_str());
        return 0;
    }

    //---------------------------------------------------------------------------------------------
    unsigned char caveNoMax = 'A' + caveMax - 1;
    // caveNoMax = 'A';  // one cave
    for(unsigned char caveNo='A'; caveNo<=caveNoMax; caveNo++)
    // for(unsigned char caveNo='J'; caveNo<='J'; caveNo++)
    {
        bool cok = false;
        bool pok = false;
        memset(cave, 0x07, sizeof(cave));   // steelwall 
        cidx = 0;
        memset(para, 0xFF, sizeof(para));   // end of cave
        pidx = 0;

        char caveNoBuf[2] = {0};
        caveNoBuf[0] = caveNo;

        // filename = path + fileprefix + "--cave--" + string(caveNoBuf) + ".raw";  // auf dem directory 
        filename = path + fileprefix + "\\" + fileprefix + "--cave--" + string(caveNoBuf) + ".raw";  // auf eigenem dir
        printf("------ %s\n", filename.c_str());
        FILE* fpw = NULL;

        FILE* fp = fopen(filename.c_str(), "rb");
        if (fp!=NULL){
            cok = true;
            while(!feof(fp)){
                int rx  = 0;
                unsigned char buf[2] = {0};
                rx = fread(buf,1,1,fp);
                if (rx == 0 || rx>=0xFFF0){
                    break;
                }
                unsigned char byte;
                byte = buf[0];

                cave[cidx] = byte;
                cidx++;
         
                if (cidx >= maxread) {
                    break;
                }
            }
            fclose(fp);
        }

        // filename = path + fileprefix + "--para--" + string(caveNoBuf) + ".raw";
        filename = path + fileprefix + "\\" + fileprefix + "--para--" + string(caveNoBuf) + ".raw";
        fp = fopen(filename.c_str(), "rb");
        if (fp!=NULL){
            pok = true;
            while(!feof(fp)){
                int rx  = 0;
                unsigned char buf[2] = {0};
                rx = fread(buf,1,1,fp);
                if (rx == 0 || rx>=0xFFF0){
                    break;
                }
                unsigned char byte;
                byte = buf[0];

                para[pidx] = byte;
                pidx++;

                if (pidx >= maxpara) {
                    break;
                }
            }
            fclose(fp);
        }

        // all ok output
        if (cok && pok)
        {
            printf("\n\n");

            // write file -- create file with header
            char wfile[256];
            sprintf(wfile, "%s\\%2.2d.WLC", string(path + fileprefix).c_str(), caveNo-'A'+1);
            fpw = fopen (wfile, "wb");
            if (fpw != NULL) {
                unsigned char wbuf[] = {0x00, 0xC1, 'W', 'L', 'C'};
                if (doDecodeCave == 1) {
                    wbuf[4] = '2';  //BD02 
                }
                fwrite(wbuf, 1, sizeof(wbuf), fpw);
            }
 
            // cave ausgeben
            int cc = 0;
            printf("%8.8X  ", cc);
            for (int yy=1;yy<=22;yy++) {
                for (int xx=1;xx<=40;xx++) {
                    printf("%c", GetCreatorCode(cave[cc]));
                    // schreiben                
                    if (fpw != NULL) {
                        unsigned char wbuf[] = {cave[cc]};
                        fwrite(wbuf, 1, sizeof(wbuf), fpw);
                    }
                    cc++;
                }
                printf("\n");
                printf("%8.8X  ", cc);
            }
            printf("\n");
            printf("\n");

            // write file -- write filler
            if (fpw != NULL) {
                unsigned char wbuf[0x10-5] = {0};
                fwrite(wbuf, 1, sizeof(wbuf), fpw);
            }

            // para + command -> ausgeben
            int caveDataIndex  = 0;
            //
            outByte(0, true, 0x2400);
            // para
            for (caveDataIndex=0; caveDataIndex<offpara; caveDataIndex++) {
                outByte(para[caveDataIndex]);
                // schreiben                
                if (fpw != NULL) {
                    unsigned char wbuf[] = {para[caveDataIndex]};
                    fwrite(wbuf, 1, sizeof(wbuf), fpw);
                }
            }
            int spara = 0x2400 + offpara;
            // command
            for (caveDataIndex = offpara; caveDataIndex <= maxpara && para[caveDataIndex] != 0xFF; caveDataIndex++) 
            {
                printf("\n%8.8X  ", spara);

                UBYTE theCode     = para[caveDataIndex];
                UBYTE cmd         = decodeCave->GetCommand(theCode);
                UBYTE cmdLen      = decodeCave->GetCommandLength(cmd);
                UBYTE cmdExtraLen = 0;

                for (int c=0; c < cmdLen; c++) {
                    UBYTE byte = para[caveDataIndex];
                    printf("%2.2X ", byte);
                    if (fpw != NULL) {
                        unsigned char wbuf[] = {byte};
                        fwrite(wbuf, 1, sizeof(wbuf), fpw);
                    }
                    if (doDecodeCave == 1) {
                        if (theCode == 0x05 && c == 2) {
                            cmdExtraLen = byte;
                        }
                    }
                    if (c < cmdLen-1) {
                        caveDataIndex++;
                    }
                }

                printf("%*.*s    theCode: %2.2X cmd: %2.2X cmdLen: %d cmdExtraLen: %d", 30-cmdLen*3, 30-cmdLen*3, " ", theCode, cmd, cmdLen, cmdExtraLen);
                spara += cmdLen;

                if (cmdExtraLen > 0) {
                    caveDataIndex++;
                    printf("\n%8.8X  ", spara);
                }
                for (int c=0; c < cmdExtraLen; c++) {
                    UBYTE byte = para[caveDataIndex];
                    printf("%2.2X ", byte);
                    if (fpw != NULL) {
                        unsigned char wbuf[] = {byte};
                        fwrite(wbuf, 1, sizeof(wbuf), fpw);
                    }
                    if (c < cmdExtraLen-1) {
                        caveDataIndex++;
                    }
                }
                if (cmdExtraLen > 0) {
                    spara += cmdExtraLen;
                }
            }
            printf("\n%8.8X  %2.2X    %27s end of cave\n", spara, 0xFF, " ");  // end of cave
            // schreiben                
            if (fpw != NULL) {
                unsigned char wbuf[] = {0xFF};
                fwrite(wbuf, 1, sizeof(wbuf), fpw);
            }

            // write file -- write filler
            if (fpw != NULL) {
                if (caveDataIndex > offpara) {
                    int fillb = 15 - (caveDataIndex%16);
                    if (fillb == 0) {
                        fillb = 16;
                    }
                    for(int b=0; b<fillb; b++) {
                        unsigned char wbuf[] = {0xFF};
                        fwrite(wbuf, 1, sizeof(wbuf), fpw);
                    }
                }
                fclose(fpw);
            }

            printf("\n");
            printf("\n");
            //===================================================================================================================== 
            // para with descr output
            theFile = stdout;
            unsigned char* aCaveData = &para[0];  // pointer to para
            switch (doDecodeCave) {
                case 0 : decodeCave->DoDecodeCave(caveNo, aCaveData, offpara, NULL);  break;
                case 1 : decodeCave->DoDecodeCave(caveNo, aCaveData, offpara, &prog[0]);  break;
                default: doDecodeCave = -1; break;
            }
            printf("\n");
            printf("\n");
            //===================================================================================================================== 
            aCaveData = GetRealCaveDataPointer(0,2);  // &realCaveData[2*MAXX+0];
            if (doDecodeCave >= 0) {
                if (memcmp(aCaveData, cave, 40*22) == 0) {
                    printf("-I- equal: %s\n", caveNoBuf);
                } else {
                    printf("-W- NOT equal: %s\n", caveNoBuf);
                }
            }
        } else {
            break;
        }
    }

    printf("\n");
    printf("\n");
}


